/*
  Copyright (C) 2002-2019 CERN for the benefit of the ATLAS collaboration
*/

#include <mutex>

#include "FaserCaloSimEvent/CaloHitIdHelper.h"
#include "StoreGate/StoreGateSvc.h"
#include "StoreGate/StoreGateSvc.h"
#include "FaserCaloIdentifier/EcalID.h"

#include "G4Types.hh"
#ifdef G4MULTITHREADED
#  include "GaudiKernel/ContextSpecificPtr.h"
#endif

static std::mutex sgMutex;

// This class is singleton and static method and variable are used.
ATLAS_NO_CHECK_FILE_THREAD_SAFETY;

//
// private constructor
CaloHitIdHelper::CaloHitIdHelper() :HitIdHelper() {
  Initialize();
}

CaloHitIdHelper* CaloHitIdHelper::GetHelper() {
#ifdef G4MULTITHREADED
  // Context-specific singleton
  static Gaudi::Hive::ContextSpecificPtr<CaloHitIdHelper> helperPtr;
  if(!helperPtr) helperPtr = new CaloHitIdHelper();
  return helperPtr.get();
#else
  static CaloHitIdHelper helper;
  return &helper;
#endif
}

void CaloHitIdHelper::Initialize() {

  // determine whether hits were created with an SLHC dictionary
  // in which case eta module field is expanded.
  // Need to lock this thread-unsafe retrieval
  const EcalID* pix;
  ServiceHandle<StoreGateSvc> detStore ("DetectorStore", "CaloHitIdHelper");
  if (detStore.retrieve().isSuccess()) {
    if (detStore->retrieve(pix, "EcalID").isFailure()) { pix = 0; }
  }

  InitializeField("Row", 0, 1);
  InitializeField("Module", 0, 1);
}

// Station
int CaloHitIdHelper::getRow(const int& hid) const
{
  return this->GetFieldValue("Row", hid);
}

// Plate
int CaloHitIdHelper::getModule(const int& hid) const
{
  return this->GetFieldValue("Module", hid);
}

//
// Info packing:
int CaloHitIdHelper::buildHitId( const int row, 
                                 const int module) const
{
  int theID(0);
  this->SetFieldValue("Row",     row,    theID);
  this->SetFieldValue("Module",  module, theID);
  return theID;
}
