# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

#!/usr/bin/env python
import sys
from AthenaConfiguration.ComponentAccumulator import ComponentAccumulator
from AthenaConfiguration.ComponentFactory import CompFactory

def ClusterReadExampleCfg(flags, **kwargs):
    """Return ComponentAccumulator for ClusterReadAlg"""
    acc = ComponentAccumulator()
    # clusterTool = acc.popToolsAndMerge(FaserSCT_ClusterizationToolCfg(flags))
    # kwargs.setdefault("SCT_ClusteringTool", clusterTool)
    # kwargs.setdefault("DataObjectName", "SCT_RDOs")
    # kwargs.setdefault("ClustersName", "SCT_ClusterContainer")
    # kwargs.setdefault("SCT_FlaggedCondData", "SCT_Flags")
    # kwargs.setdefault("OutputSDOName", "SCT_SDO_Map")
    Tracker__ClusterReadAlg=CompFactory.ClusterReadAlg
    acc.addEventAlgo(Tracker__ClusterReadAlg(**kwargs))
    
    thistSvc = CompFactory.THistSvc()
    thistSvc.Output += ["HIST DATAFILE='ClusterReadHistograms.root' OPT='RECREATE'"]
    acc.addService(thistSvc)
    return acc


if __name__ == "__main__":
    from AthenaCommon.Configurable import Configurable
    from CalypsoConfiguration.AllConfigFlags import ConfigFlags

    Configurable.configurableRun3Behavior = True
    
# Flags for this job
    ConfigFlags.Input.isMC = True                                # Needed to bypass autoconfig
    ConfigFlags.IOVDb.GlobalTag = "OFLCOND-FASER-01"          # Needed to bypass autoconfig, only the "OFLCOND" matters at the moment
    ConfigFlags.GeoModel.FaserVersion     = "FASER-01"           # Default FASER geometry
    #ConfigFlags.Detector.SimulateFaserSCT = True
    ConfigFlags.Input.Files = ["run001332.ESD.pool.root"] #["cosmics.HITS.pool.root"]#["my.COSMICHITS.pool.root"]
    ConfigFlags.lock()

# Configure components
    from CalypsoConfiguration.MainServicesConfig import MainServicesCfg
    from AthenaPoolCnvSvc.PoolReadConfig import PoolReadCfg
    acc = MainServicesCfg(ConfigFlags)
    acc.merge(PoolReadCfg(ConfigFlags))

# Set things up to create a conditions DB with neutral Tracker alignment transforms
    acc.merge(ClusterReadExampleCfg(ConfigFlags))

# Execute and finish
    sys.exit(int(acc.run(maxEvents=-1).isFailure()))
